"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.withUpdateMintMetadata = exports.withRevokeEntry = exports.withRevokeReverseEntry = exports.withSetReverseEntry = exports.withSetEntryData = exports.withClaimEntry = exports.withUpdateClaimRequest = exports.withCreateClaimRequest = exports.withInitEntry = exports.withUpdateNamespace = exports.withCreateNamespace = exports.withInit = void 0;
const tslib_1 = require("tslib");
const certificate = tslib_1.__importStar(require("@cardinal/certificates"));
const certificates_1 = require("@cardinal/certificates");
const mplTokenMetadata = tslib_1.__importStar(require("@metaplex-foundation/mpl-token-metadata"));
const anchor = tslib_1.__importStar(require("@project-serum/anchor"));
const splToken = tslib_1.__importStar(require("@solana/spl-token"));
const web3 = tslib_1.__importStar(require("@solana/web3.js"));
const _1 = require(".");
function withInit(connection, wallet, rentalPercentage, transaction) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const provider = new anchor.Provider(connection, wallet, {});
        const namespacesProgram = new anchor.Program(_1.NAMESPACES_IDL, _1.NAMESPACES_PROGRAM_ID, provider);
        const [globalContextId, bump] = yield web3.PublicKey.findProgramAddress([anchor.utils.bytes.utf8.encode(_1.GLOBAL_CONTEXT_SEED)], namespacesProgram.programId);
        transaction.add(namespacesProgram.instruction.init({
            rentPercentage: new anchor.BN(rentalPercentage),
            bump,
        }, {
            accounts: {
                globalContext: globalContextId,
                authority: provider.wallet.publicKey,
                payer: provider.wallet.publicKey,
                systemProgram: anchor.web3.SystemProgram.programId,
            },
        }));
        return transaction;
    });
}
exports.withInit = withInit;
function withCreateNamespace(connection, wallet, name, updateAuthority, rentAuthority, approveAuthority, schema, paymentAmountDaily, paymentMint, minRentalSeconds, maxRentalSeconds, transferableEntries, transaction) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const provider = new anchor.Provider(connection, wallet, {});
        const namespacesProgram = new anchor.Program(_1.NAMESPACES_IDL, _1.NAMESPACES_PROGRAM_ID, provider);
        const [namespaceId, bump] = yield web3.PublicKey.findProgramAddress([
            anchor.utils.bytes.utf8.encode(_1.NAMESPACE_SEED),
            anchor.utils.bytes.utf8.encode(name),
        ], namespacesProgram.programId);
        transaction.add(namespacesProgram.instruction.createNamespace({
            bump,
            name,
            updateAuthority,
            rentAuthority,
            approveAuthority,
            schema,
            paymentAmountDaily,
            paymentMint,
            minRentalSeconds,
            maxRentalSeconds,
            transferableEntries,
        }, {
            accounts: {
                namespace: namespaceId,
                authority: provider.wallet.publicKey,
                payer: provider.wallet.publicKey,
                systemProgram: anchor.web3.SystemProgram.programId,
            },
        }));
        return transaction;
    });
}
exports.withCreateNamespace = withCreateNamespace;
function withUpdateNamespace(connection, wallet, name, updateAuthority, rentAuthority, approveAuthority, paymentAmountDaily, paymentMint, minRentalSeconds, maxRentalSeconds, transferableEntries, transaction) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const provider = new anchor.Provider(connection, wallet, {});
        const namespacesProgram = new anchor.Program(_1.NAMESPACES_IDL, _1.NAMESPACES_PROGRAM_ID, provider);
        const [namespaceId] = yield web3.PublicKey.findProgramAddress([
            anchor.utils.bytes.utf8.encode(_1.NAMESPACE_SEED),
            anchor.utils.bytes.utf8.encode(name),
        ], namespacesProgram.programId);
        transaction.add(namespacesProgram.instruction.updateNamespace({
            updateAuthority,
            rentAuthority,
            approveAuthority,
            paymentAmountDaily,
            paymentMint,
            minRentalSeconds,
            maxRentalSeconds,
            transferableEntries,
        }, {
            accounts: {
                namespace: namespaceId,
                updateAuthority: provider.wallet.publicKey,
            },
        }));
        return transaction;
    });
}
exports.withUpdateNamespace = withUpdateNamespace;
function withInitEntry(connection, wallet, certificateMintId, namespaceName, entryName, transaction) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const provider = new anchor.Provider(connection, wallet, {});
        const namespacesProgram = new anchor.Program(_1.NAMESPACES_IDL, _1.NAMESPACES_PROGRAM_ID, provider);
        const [namespaceId] = yield web3.PublicKey.findProgramAddress([
            anchor.utils.bytes.utf8.encode(_1.NAMESPACE_SEED),
            anchor.utils.bytes.utf8.encode(namespaceName),
        ], namespacesProgram.programId);
        const [entryId, entryBump] = yield web3.PublicKey.findProgramAddress([
            anchor.utils.bytes.utf8.encode(_1.ENTRY_SEED),
            namespaceId.toBytes(),
            anchor.utils.bytes.utf8.encode(entryName),
        ], namespacesProgram.programId);
        const [mintManagerId, mintManagerBump] = yield web3.PublicKey.findProgramAddress([
            anchor.utils.bytes.utf8.encode(certificates_1.MINT_MANAGER_SEED),
            certificateMintId.toBytes(),
        ], certificates_1.CERTIFICATE_PROGRAM_ID);
        const [certificateMintMetadataId] = yield web3.PublicKey.findProgramAddress([
            Buffer.from(mplTokenMetadata.MetadataProgram.PREFIX),
            mplTokenMetadata.MetadataProgram.PUBKEY.toBuffer(),
            certificateMintId.toBuffer(),
        ], mplTokenMetadata.MetadataProgram.PUBKEY);
        const mintBalanceNeeded = yield splToken.Token.getMinBalanceRentForExemptMint(provider.connection);
        transaction.add(web3.SystemProgram.createAccount({
            fromPubkey: provider.wallet.publicKey,
            newAccountPubkey: certificateMintId,
            lamports: mintBalanceNeeded,
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access
            space: splToken.MintLayout.span,
            programId: splToken.TOKEN_PROGRAM_ID,
        }));
        const namespaceCertificateTokenAccountId = yield splToken.Token.getAssociatedTokenAddress(splToken.ASSOCIATED_TOKEN_PROGRAM_ID, splToken.TOKEN_PROGRAM_ID, certificateMintId, namespaceId, true);
        transaction.add(namespacesProgram.instruction.initEntry({
            name: entryName,
            entryBump,
            mintManagerBump,
        }, {
            accounts: {
                namespace: namespaceId,
                entry: entryId,
                payer: provider.wallet.publicKey,
                namespaceCertificateTokenAccount: namespaceCertificateTokenAccountId,
                // cpi
                mintManager: mintManagerId,
                certificateMint: certificateMintId,
                certificateMintMetadata: certificateMintMetadataId,
                // programs
                certificateProgram: certificates_1.CERTIFICATE_PROGRAM_ID,
                tokenMetadataProgram: mplTokenMetadata.MetadataProgram.PUBKEY,
                tokenProgram: splToken.TOKEN_PROGRAM_ID,
                associatedToken: splToken.ASSOCIATED_TOKEN_PROGRAM_ID,
                rent: web3.SYSVAR_RENT_PUBKEY,
                systemProgram: anchor.web3.SystemProgram.programId,
            },
        }));
        return transaction;
    });
}
exports.withInitEntry = withInitEntry;
function withCreateClaimRequest(connection, wallet, namespaceName, entryName, user, transaction) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const provider = new anchor.Provider(connection, wallet, {});
        const namespacesProgram = new anchor.Program(_1.NAMESPACES_IDL, _1.NAMESPACES_PROGRAM_ID, provider);
        const [namespaceId] = yield web3.PublicKey.findProgramAddress([
            anchor.utils.bytes.utf8.encode(_1.NAMESPACE_SEED),
            anchor.utils.bytes.utf8.encode(namespaceName),
        ], namespacesProgram.programId);
        const [claimRequestId, claimRequestBump] = yield web3.PublicKey.findProgramAddress([
            anchor.utils.bytes.utf8.encode(_1.CLAIM_REQUEST_SEED),
            namespaceId.toBytes(),
            anchor.utils.bytes.utf8.encode(entryName),
            user.toBytes(),
        ], namespacesProgram.programId);
        transaction.add(namespacesProgram.instruction.createClaimRequest(entryName, claimRequestBump, user, {
            accounts: {
                namespace: namespaceId,
                payer: provider.wallet.publicKey,
                claimRequest: claimRequestId,
                systemProgram: anchor.web3.SystemProgram.programId,
            },
        }));
        return transaction;
    });
}
exports.withCreateClaimRequest = withCreateClaimRequest;
function withUpdateClaimRequest(connection, wallet, namespaceName, claimRequestId, isApproved, transaction) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const provider = new anchor.Provider(connection, wallet, {});
        const namespacesProgram = new anchor.Program(_1.NAMESPACES_IDL, _1.NAMESPACES_PROGRAM_ID, provider);
        const [namespaceId] = yield web3.PublicKey.findProgramAddress([
            anchor.utils.bytes.utf8.encode(_1.NAMESPACE_SEED),
            anchor.utils.bytes.utf8.encode(namespaceName),
        ], namespacesProgram.programId);
        transaction.add(namespacesProgram.instruction.updateClaimRequest(isApproved, {
            accounts: {
                namespace: namespaceId,
                approveAuthority: provider.wallet.publicKey,
                rentRequest: claimRequestId,
            },
        }));
        return transaction;
    });
}
exports.withUpdateClaimRequest = withUpdateClaimRequest;
function withClaimEntry(connection, wallet, namespaceName, entryName, certificateMintId, duration, transaction) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const provider = new anchor.Provider(connection, wallet, {});
        const namespacesProgram = new anchor.Program(_1.NAMESPACES_IDL, _1.NAMESPACES_PROGRAM_ID, provider);
        const [namespaceId] = yield web3.PublicKey.findProgramAddress([
            anchor.utils.bytes.utf8.encode(_1.NAMESPACE_SEED),
            anchor.utils.bytes.utf8.encode(namespaceName),
        ], namespacesProgram.programId);
        const [entryId] = yield web3.PublicKey.findProgramAddress([
            anchor.utils.bytes.utf8.encode(_1.ENTRY_SEED),
            namespaceId.toBytes(),
            anchor.utils.bytes.utf8.encode(entryName),
        ], namespacesProgram.programId);
        const [claimRequestId] = yield web3.PublicKey.findProgramAddress([
            anchor.utils.bytes.utf8.encode(_1.CLAIM_REQUEST_SEED),
            namespaceId.toBytes(),
            anchor.utils.bytes.utf8.encode(entryName),
            provider.wallet.publicKey.toBytes(),
        ], namespacesProgram.programId);
        const namespace = yield namespacesProgram.account.namespace.fetch(namespaceId);
        const [certificateId, certificateBump] = yield web3.PublicKey.findProgramAddress([
            anchor.utils.bytes.utf8.encode(certificates_1.CERTIFICATE_SEED),
            certificateMintId.toBuffer(),
        ], certificates_1.CERTIFICATE_PROGRAM_ID);
        const [mintManagerId] = yield web3.PublicKey.findProgramAddress([
            anchor.utils.bytes.utf8.encode(certificates_1.MINT_MANAGER_SEED),
            certificateMintId.toBytes(),
        ], certificates_1.CERTIFICATE_PROGRAM_ID);
        const namespaceCertificateTokenAccountId = yield splToken.Token.getAssociatedTokenAddress(splToken.ASSOCIATED_TOKEN_PROGRAM_ID, splToken.TOKEN_PROGRAM_ID, certificateMintId, namespaceId, true);
        const certificatePaymentTokenAccountId = yield (0, certificates_1.withFindOrInitAssociatedTokenAccount)(transaction, provider.connection, namespace.paymentMint, certificateId, provider.wallet.publicKey, true);
        const userCertificateTokenAccountId = yield (0, certificates_1.withFindOrInitAssociatedTokenAccount)(transaction, provider.connection, certificateMintId, provider.wallet.publicKey, provider.wallet.publicKey);
        const userPaymentTokenAccountId = yield (0, certificates_1.withFindOrInitAssociatedTokenAccount)(transaction, provider.connection, namespace.paymentMint, provider.wallet.publicKey, provider.wallet.publicKey);
        const certificateTokenAccountId = yield splToken.Token.getAssociatedTokenAddress(splToken.ASSOCIATED_TOKEN_PROGRAM_ID, splToken.TOKEN_PROGRAM_ID, certificateMintId, certificateId, true);
        transaction.add(namespacesProgram.instruction.claimEntry({
            duration: duration > 0 ? new anchor.BN(duration) : null,
            certificateBump,
        }, {
            accounts: {
                namespace: namespaceId,
                entry: entryId,
                user: provider.wallet.publicKey,
                payer: provider.wallet.publicKey,
                paymentMint: namespace.paymentMint,
                claimRequest: claimRequestId,
                // CPI accounts
                mintManager: mintManagerId,
                certificate: certificateId,
                certificateMint: certificateMintId,
                certificateTokenAccount: certificateTokenAccountId,
                certificatePaymentTokenAccount: certificatePaymentTokenAccountId,
                userCertificateTokenAccount: userCertificateTokenAccountId,
                userPaymentTokenAccount: userPaymentTokenAccountId,
                namespaceCertificateTokenAccount: namespaceCertificateTokenAccountId,
                // programs
                certificateProgram: certificates_1.CERTIFICATE_PROGRAM_ID,
                tokenProgram: splToken.TOKEN_PROGRAM_ID,
                associatedToken: splToken.ASSOCIATED_TOKEN_PROGRAM_ID,
                rent: web3.SYSVAR_RENT_PUBKEY,
                systemProgram: anchor.web3.SystemProgram.programId,
            },
        }));
        return transaction;
    });
}
exports.withClaimEntry = withClaimEntry;
function withSetEntryData(connection, wallet, namespaceName, entryName, entryData, transaction) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const provider = new anchor.Provider(connection, wallet, {});
        const namespacesProgram = new anchor.Program(_1.NAMESPACES_IDL, _1.NAMESPACES_PROGRAM_ID, provider);
        const [namespaceId] = yield web3.PublicKey.findProgramAddress([
            anchor.utils.bytes.utf8.encode(_1.NAMESPACE_SEED),
            anchor.utils.bytes.utf8.encode(namespaceName),
        ], namespacesProgram.programId);
        const [entryId] = yield web3.PublicKey.findProgramAddress([
            anchor.utils.bytes.utf8.encode(_1.ENTRY_SEED),
            namespaceId.toBytes(),
            anchor.utils.bytes.utf8.encode(entryName),
        ], namespacesProgram.programId);
        const entry = yield namespacesProgram.account.entry.fetch(entryId);
        const [certificateId] = yield certificate.certificateIdForMint(entry.mint);
        const userCertificateTokenAccountId = yield (0, certificates_1.withFindOrInitAssociatedTokenAccount)(transaction, provider.connection, entry.mint, provider.wallet.publicKey, provider.wallet.publicKey);
        transaction.add(namespacesProgram.instruction.setEntryData(entryData, {
            accounts: {
                namespace: namespaceId,
                entry: entryId,
                userCertificateTokenAccount: userCertificateTokenAccountId,
                certificate: certificateId,
                user: provider.wallet.publicKey,
                payer: provider.wallet.publicKey,
                systemProgram: anchor.web3.SystemProgram.programId,
            },
        }));
        return transaction;
    });
}
exports.withSetEntryData = withSetEntryData;
function withSetReverseEntry(connection, wallet, namespaceName, entryName, certificateMintId, transaction) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const provider = new anchor.Provider(connection, wallet, {});
        const namespacesProgram = new anchor.Program(_1.NAMESPACES_IDL, _1.NAMESPACES_PROGRAM_ID, provider);
        const [namespaceId] = yield web3.PublicKey.findProgramAddress([
            anchor.utils.bytes.utf8.encode(_1.NAMESPACE_SEED),
            anchor.utils.bytes.utf8.encode(namespaceName),
        ], namespacesProgram.programId);
        const [entryId] = yield web3.PublicKey.findProgramAddress([
            anchor.utils.bytes.utf8.encode(_1.ENTRY_SEED),
            namespaceId.toBytes(),
            anchor.utils.bytes.utf8.encode(entryName),
        ], namespacesProgram.programId);
        const [reverseEntryId, reverseEntryBump] = yield web3.PublicKey.findProgramAddress([
            anchor.utils.bytes.utf8.encode(_1.REVERSE_ENTRY_SEED),
            wallet.publicKey.toBytes(),
        ], namespacesProgram.programId);
        const [certificateId] = yield certificate.certificateIdForMint(certificateMintId);
        const userCertificateTokenAccountId = yield splToken.Token.getAssociatedTokenAddress(splToken.ASSOCIATED_TOKEN_PROGRAM_ID, splToken.TOKEN_PROGRAM_ID, certificateMintId, provider.wallet.publicKey);
        transaction.add(namespacesProgram.instruction.setReverseEntry(reverseEntryBump, {
            accounts: {
                namespace: namespaceId,
                entry: entryId,
                reverseEntry: reverseEntryId,
                userCertificateTokenAccount: userCertificateTokenAccountId,
                certificate: certificateId,
                user: provider.wallet.publicKey,
                payer: provider.wallet.publicKey,
                systemProgram: anchor.web3.SystemProgram.programId,
            },
        }));
        return transaction;
    });
}
exports.withSetReverseEntry = withSetReverseEntry;
function withRevokeReverseEntry(connection, wallet, namespaceName, entryName, reverseEntryId, claimRequestId, transaction) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const provider = new anchor.Provider(connection, wallet, {});
        const namespacesProgram = new anchor.Program(_1.NAMESPACES_IDL, _1.NAMESPACES_PROGRAM_ID, provider);
        const [namespaceId] = yield web3.PublicKey.findProgramAddress([
            anchor.utils.bytes.utf8.encode(_1.NAMESPACE_SEED),
            anchor.utils.bytes.utf8.encode(namespaceName),
        ], namespacesProgram.programId);
        const [entryId] = yield web3.PublicKey.findProgramAddress([
            anchor.utils.bytes.utf8.encode(_1.ENTRY_SEED),
            namespaceId.toBytes(),
            anchor.utils.bytes.utf8.encode(entryName),
        ], namespacesProgram.programId);
        transaction.add(namespacesProgram.instruction.revokeReverseEntry({
            accounts: {
                namespace: namespaceId,
                entry: entryId,
                reverseEntry: reverseEntryId,
                claimRequest: claimRequestId,
                invalidator: wallet.publicKey,
            },
        }));
        return transaction;
    });
}
exports.withRevokeReverseEntry = withRevokeReverseEntry;
function withRevokeEntry(connection, wallet, namespaceName, entryName, certificateMintId, certificateOwnerId, claimRequestId, transaction) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const provider = new anchor.Provider(connection, wallet, {});
        const namespacesProgram = new anchor.Program(_1.NAMESPACES_IDL, _1.NAMESPACES_PROGRAM_ID, provider);
        const [namespaceId] = yield web3.PublicKey.findProgramAddress([
            anchor.utils.bytes.utf8.encode(_1.NAMESPACE_SEED),
            anchor.utils.bytes.utf8.encode(namespaceName),
        ], namespacesProgram.programId);
        const [entryId] = yield web3.PublicKey.findProgramAddress([
            anchor.utils.bytes.utf8.encode(_1.ENTRY_SEED),
            namespaceId.toBytes(),
            anchor.utils.bytes.utf8.encode(entryName),
        ], namespacesProgram.programId);
        const namespace = yield namespacesProgram.account.namespace.fetch(namespaceId);
        const [certificateId] = yield web3.PublicKey.findProgramAddress([
            anchor.utils.bytes.utf8.encode(certificates_1.CERTIFICATE_SEED),
            certificateMintId.toBuffer(),
        ], certificates_1.CERTIFICATE_PROGRAM_ID);
        const [mintManagerId] = yield web3.PublicKey.findProgramAddress([
            anchor.utils.bytes.utf8.encode(certificates_1.MINT_MANAGER_SEED),
            certificateMintId.toBytes(),
        ], certificates_1.CERTIFICATE_PROGRAM_ID);
        const namespaceCertificateTokenAccountId = yield splToken.Token.getAssociatedTokenAddress(splToken.ASSOCIATED_TOKEN_PROGRAM_ID, splToken.TOKEN_PROGRAM_ID, certificateMintId, namespaceId, true);
        const namespacePaymentTokenAccountId = yield (0, certificates_1.withFindOrInitAssociatedTokenAccount)(transaction, provider.connection, namespace.paymentMint, namespaceId, provider.wallet.publicKey, true);
        const certificatePaymentTokenAccountId = yield splToken.Token.getAssociatedTokenAddress(splToken.ASSOCIATED_TOKEN_PROGRAM_ID, splToken.TOKEN_PROGRAM_ID, namespace.paymentMint, certificateId, true);
        const userCertificateTokenAccountId = yield splToken.Token.getAssociatedTokenAddress(splToken.ASSOCIATED_TOKEN_PROGRAM_ID, splToken.TOKEN_PROGRAM_ID, certificateMintId, certificateOwnerId);
        const userPaymentTokenAccountId = yield (0, certificates_1.withFindOrInitAssociatedTokenAccount)(transaction, provider.connection, namespace.paymentMint, certificateOwnerId, provider.wallet.publicKey);
        const certificateTokenAccountId = yield splToken.Token.getAssociatedTokenAddress(splToken.ASSOCIATED_TOKEN_PROGRAM_ID, splToken.TOKEN_PROGRAM_ID, certificateMintId, certificateId, true);
        transaction.add(namespacesProgram.instruction.revokeEntry({
            accounts: {
                namespace: namespaceId,
                entry: entryId,
                claimRequest: claimRequestId,
                namespaceCertificateTokenAccount: namespaceCertificateTokenAccountId,
                namespacePaymentTokenAccount: namespacePaymentTokenAccountId,
                invalidator: provider.wallet.publicKey,
                // CPI accounts
                mintManager: mintManagerId,
                certificate: certificateId,
                certificateMint: certificateMintId,
                certificateTokenAccount: certificateTokenAccountId,
                certificatePaymentTokenAccount: certificatePaymentTokenAccountId,
                userCertificateTokenAccount: userCertificateTokenAccountId,
                userPaymentTokenAccount: userPaymentTokenAccountId,
                // programs
                certificateProgram: certificates_1.CERTIFICATE_PROGRAM_ID,
                tokenProgram: splToken.TOKEN_PROGRAM_ID,
            },
        }));
        return transaction;
    });
}
exports.withRevokeEntry = withRevokeEntry;
function withUpdateMintMetadata(connection, wallet, namespaceId, entryId, certificateMintId, transaction) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const provider = new anchor.Provider(connection, wallet, {});
        const namespacesProgram = new anchor.Program(_1.NAMESPACES_IDL, _1.NAMESPACES_PROGRAM_ID, provider);
        const [certificateMintMetadataId] = yield web3.PublicKey.findProgramAddress([
            anchor.utils.bytes.utf8.encode(mplTokenMetadata.MetadataProgram.PREFIX),
            mplTokenMetadata.MetadataProgram.PUBKEY.toBuffer(),
            certificateMintId.toBuffer(),
        ], mplTokenMetadata.MetadataProgram.PUBKEY);
        transaction.add(namespacesProgram.instruction.updateEntryMintMetadata({
            accounts: {
                namespace: namespaceId,
                approveAuthority: provider.wallet.publicKey,
                entry: entryId,
                certificateMintMetadata: certificateMintMetadataId,
                tokenMetadataProgram: mplTokenMetadata.MetadataProgram.PUBKEY,
            },
        }));
        return transaction;
    });
}
exports.withUpdateMintMetadata = withUpdateMintMetadata;
//# sourceMappingURL=instructions.js.map