"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.reverseEntryId = exports.nameEntryId = exports.claimRequestId = exports.nameForDisplay = exports.tryGetName = exports.displayAddress = exports.shortenAddress = exports.breakName = exports.formatName = void 0;
const tslib_1 = require("tslib");
const namespaces_1 = require("@cardinal/namespaces");
const anchor = tslib_1.__importStar(require("@project-serum/anchor"));
const web3 = tslib_1.__importStar(require("@solana/web3.js"));
const accounts_1 = require("./accounts");
const constants_1 = require("./constants");
function formatName(namespace, name) {
    return namespace === "twitter" ? `@${name}` : `${name}.${namespace}`;
}
exports.formatName = formatName;
function breakName(fullName) {
    if (fullName.startsWith("@")) {
        return ["twitter", fullName.split("@")[1]];
    }
    const [entryName, namespace] = fullName.split(".");
    return [namespace, entryName];
}
exports.breakName = breakName;
/**
 * shorten the checksummed version of the input address to have 4 characters at start and end
 * @param address
 * @param chars
 * @returns
 */
function shortenAddress(address, chars = 5) {
    return `${address.substring(0, chars)}...${address.substring(address.length - chars)}`;
}
exports.shortenAddress = shortenAddress;
function displayAddress(address, shorten = true) {
    return shorten ? shortenAddress(address) : address;
}
exports.displayAddress = displayAddress;
function tryGetName(connection, pubkey) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        try {
            const reverseEntry = yield (0, accounts_1.getReverseEntry)(connection, pubkey);
            return formatName(
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore
            reverseEntry.parsed.namespaceName, reverseEntry.parsed.entryName);
        }
        catch (e) {
            console.log(e);
        }
        return undefined;
    });
}
exports.tryGetName = tryGetName;
function nameForDisplay(connection, pubkey) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const name = yield tryGetName(connection, pubkey);
        return name || displayAddress(pubkey.toString());
    });
}
exports.nameForDisplay = nameForDisplay;
function claimRequestId(namespaceName, entryName, user) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const [namespaceId] = yield web3.PublicKey.findProgramAddress([
            anchor.utils.bytes.utf8.encode(constants_1.NAMESPACE_SEED),
            anchor.utils.bytes.utf8.encode(namespaceName),
        ], namespaces_1.NAMESPACES_PROGRAM_ID);
        return web3.PublicKey.findProgramAddress([
            anchor.utils.bytes.utf8.encode(constants_1.CLAIM_REQUEST_SEED),
            namespaceId.toBytes(),
            anchor.utils.bytes.utf8.encode(entryName),
            user.toBytes(),
        ], namespaces_1.NAMESPACES_PROGRAM_ID);
    });
}
exports.claimRequestId = claimRequestId;
function nameEntryId(namespaceName, entryName) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const [namespaceId] = yield web3.PublicKey.findProgramAddress([
            anchor.utils.bytes.utf8.encode(constants_1.NAMESPACE_SEED),
            anchor.utils.bytes.utf8.encode(namespaceName),
        ], namespaces_1.NAMESPACES_PROGRAM_ID);
        return web3.PublicKey.findProgramAddress([
            anchor.utils.bytes.utf8.encode(constants_1.ENTRY_SEED),
            namespaceId.toBytes(),
            anchor.utils.bytes.utf8.encode(entryName),
        ], namespaces_1.NAMESPACES_PROGRAM_ID);
    });
}
exports.nameEntryId = nameEntryId;
const reverseEntryId = (address) => {
    return web3.PublicKey.findProgramAddress([anchor.utils.bytes.utf8.encode(constants_1.REVERSE_ENTRY_SEED), address.toBytes()], namespaces_1.NAMESPACES_PROGRAM_ID);
};
exports.reverseEntryId = reverseEntryId;
//# sourceMappingURL=utils.js.map