import { __awaiter } from "tslib";
import { NAMESPACES_PROGRAM_ID } from "@cardinal/namespaces";
import * as anchor from "@project-serum/anchor";
import * as web3 from "@solana/web3.js";
import { getReverseEntry } from "./accounts";
import { CLAIM_REQUEST_SEED, ENTRY_SEED, NAMESPACE_SEED, REVERSE_ENTRY_SEED, } from "./constants";
export function formatName(namespace, name) {
    return namespace === "twitter" ? `@${name}` : `${name}.${namespace}`;
}
export function breakName(fullName) {
    if (fullName.startsWith("@")) {
        return ["twitter", fullName.split("@")[1]];
    }
    const [entryName, namespace] = fullName.split(".");
    return [namespace, entryName];
}
/**
 * shorten the checksummed version of the input address to have 4 characters at start and end
 * @param address
 * @param chars
 * @returns
 */
export function shortenAddress(address, chars = 5) {
    return `${address.substring(0, chars)}...${address.substring(address.length - chars)}`;
}
export function displayAddress(address, shorten = true) {
    return shorten ? shortenAddress(address) : address;
}
export function tryGetName(connection, pubkey) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const reverseEntry = yield getReverseEntry(connection, pubkey);
            return formatName(
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore
            reverseEntry.parsed.namespaceName, reverseEntry.parsed.entryName);
        }
        catch (e) {
            console.log(e);
        }
        return undefined;
    });
}
export function nameForDisplay(connection, pubkey) {
    return __awaiter(this, void 0, void 0, function* () {
        const name = yield tryGetName(connection, pubkey);
        return name || displayAddress(pubkey.toString());
    });
}
export function claimRequestId(namespaceName, entryName, user) {
    return __awaiter(this, void 0, void 0, function* () {
        const [namespaceId] = yield web3.PublicKey.findProgramAddress([
            anchor.utils.bytes.utf8.encode(NAMESPACE_SEED),
            anchor.utils.bytes.utf8.encode(namespaceName),
        ], NAMESPACES_PROGRAM_ID);
        return web3.PublicKey.findProgramAddress([
            anchor.utils.bytes.utf8.encode(CLAIM_REQUEST_SEED),
            namespaceId.toBytes(),
            anchor.utils.bytes.utf8.encode(entryName),
            user.toBytes(),
        ], NAMESPACES_PROGRAM_ID);
    });
}
export function nameEntryId(namespaceName, entryName) {
    return __awaiter(this, void 0, void 0, function* () {
        const [namespaceId] = yield web3.PublicKey.findProgramAddress([
            anchor.utils.bytes.utf8.encode(NAMESPACE_SEED),
            anchor.utils.bytes.utf8.encode(namespaceName),
        ], NAMESPACES_PROGRAM_ID);
        return web3.PublicKey.findProgramAddress([
            anchor.utils.bytes.utf8.encode(ENTRY_SEED),
            namespaceId.toBytes(),
            anchor.utils.bytes.utf8.encode(entryName),
        ], NAMESPACES_PROGRAM_ID);
    });
}
export const reverseEntryId = (address) => {
    return web3.PublicKey.findProgramAddress([anchor.utils.bytes.utf8.encode(REVERSE_ENTRY_SEED), address.toBytes()], NAMESPACES_PROGRAM_ID);
};
//# sourceMappingURL=utils.js.map